@@ -245,8 +245,11 @@ def window_fixed_size(seq: Iterable[T], size: int, maxstep: int
         cursor += step
 
 
+class _Sentinel:
+    pass
 
-def iterchunks(seq, chunksize: int) -> Iterable[Tuple]:
+
+def iterchunks(seq, chunksize: int) -> Iterable[tuple]:
     """
     Returns an iterator over chunks of seq of at most `chunksize` size.
 
@@ -260,20 +263,19 @@ def iterchunks(seq, chunksize: int) -> Iterable[Tuple]:
     >>> list(iterchunks(seq, 3))
     [(0, 1, 2), (3, 4, 5), (6, 7, 8), (9, 10, 11), (12, 13, 14), (15, 16, 17), (18, 19)]
     """
-    class Sentinel(object):
-        pass
-    padded = chain(seq, repeat(Sentinel))
+
+    padded = chain(seq, repeat(_Sentinel))
     for chunk in window(padded, size=chunksize, step=chunksize):
-        if chunk[-1] is Sentinel:
-            if chunk[0] is Sentinel:
+        if chunk[-1] is _Sentinel:
+            if chunk[0] is _Sentinel:
                 break
-            yield tuple((x for x in chunk if x is not Sentinel))
+            yield tuple((x for x in chunk if x is not _Sentinel))
             break
         else:
             yield chunk
 
 
-def parse_range(start, stop:int=None, step:int=None) -> tuple[int, int, int]:
+def parse_range(start, stop: int | None = None, step: int | None = None) -> tuple[int, int, int]:
     """
     Given arguments as passed to `range`, resolved them in `(start, stop, step)`
     """
@@ -285,7 +287,7 @@ def parse_range(start, stop:int=None, step:int=None) -> tuple[int, int, int]:
     return start, stop, step
 
 
-def chunks(start:int, stop:int=None, step:int=None) -> Iterable[tuple[int, int]]:
+def chunks(start: int, stop: int = None, step: int = None) -> Iterable[tuple[int, int]]:
     """
     Returns a generator of tuples (offset, chunksize)
 
@@ -664,24 +666,89 @@ def interleave(seqs, pass_exceptions=()):
         iters = newiters
         
 
-def split_in_chunks(seq: Iterable[T], chunksize: int) -> Iterator[list[T]]:
+def chunked(iterable, n):
+    """Break *iterable* into lists of length *n*:
+
+        >>> list(chunked([1, 2, 3, 4, 5, 6], 3))
+        [[1, 2, 3], [4, 5, 6]]
+
+    By the default, the last yielded list will have fewer than *n* elements
+    if the length of *iterable* is not divisible by *n*:
+
+        >>> list(chunked([1, 2, 3, 4, 5, 6, 7, 8], 3))
+        [[1, 2, 3], [4, 5, 6], [7, 8]]
+
+    To use a fill-in value instead, see the :func:`grouper` recipe.
+
+    If the length of *iterable* is not divisible by *n* and *strict* is
+    ``True``, then ``ValueError`` will be raised before the last
+    list is yielded.
+
     """
-    splits a sequence into chunks
+    iterator = iter(partial(take, n, iter(iterable)), [])
+    if strict:
+        if n is None:
+            raise ValueError('n must not be None when using strict mode.')
+
+        def ret():
+            for chunk in iterator:
+                if len(chunk) != n:
+                    raise ValueError('iterable is not divisible by n.')
+                yield chunk
+
+        return iter(ret())
+    else:
+        return iterator
+
+
+def _reversedenum(s: Sequence[T]) -> Iterator[tuple[int, T]]:
+    lens = len(s)
+    for idx, item in enumerate(reversed(s)):
+        yield lens - 1 - idx, item
+
+
+def duplicates(s: Iterable[T]) -> Iterator[T]:
+    """
+    Find duplicates in the iterable `s`
 
     Example
     -------
 
-    >>> s = [0, 1, 2, 3, 4, 5, 6, 8, 9]
-    >>> list(split_in_chunks(s, 3))
-    [[0, 1, 2], [3, 4, 5], [6, 7, 8]]
+        >>> l = [1, 2, 3, 1, 4, 2, 5, 6]
+        >>> list(duplicates(s))
+        [2, 1]
     """
-    chunk = []
-    for i, item in enumerate(seq):
-        chunk.append(item)
-        if i % chunksize == chunksize-1:
-            yield chunk
-            chunk = []
-    # return [seq[i:i+chunksize] for i in range(0, len(seq), chunksize)]
+    seen = set()
+    for item in s:
+        if item in seen:
+            yield item
+        else:
+            seen.add(item)
+
+
+def reversed_enumerate(s: Sequence[T]) -> Iterator[tuple[int, T]]:
+    """
+    The same as enumerate but reverses the sequence
+
+    Args:
+        s: the sequence to be reversed (needs to be finite)
+
+    Returns:
+        an iterator over tuples (index, item) on reversed
+        order
+
+    Example
+    ~~~~~~~
+
+        >>> seq = "a b c d e f".split()
+        >>> list(reversed_enumerate(seq))
+        [(5, 'f'), (4, 'e'), (3, 'd'), (2, 'c'), (1, 'b'), (0, 'a')]
+
+    """
+    # This is faster than the version above
+    enum = list(zip(range(len(s)), s))
+    enum.reverse()
+    return enum
 
 
 def classify(s: Sequence[T], keyfunc: Callable[[T], T2]) -> dict[T2, list[T]]:
