@@ -16,6 +16,7 @@ from math import gcd, sqrt, cos, sin, radians, ceil, hypot, pi, asin, floor, fac
 import sys as _sys
 import numpy as np
 from numbers import Rational, Number
+from typing import TYPE_CHECKING, TypeVar, Sequence, Callable
 
 
 try:
@@ -23,10 +24,9 @@ try:
 except ImportError:
     from fractions import Fraction
 
-from typing import TYPE_CHECKING
 if TYPE_CHECKING:
-    from typing import TypeVar
-    number_t = Rational | float
+    from typing import Union, Iterator
+    number_t = Union[Rational, float]
     T = TypeVar("T", bound=number_t)
     T2 = TypeVar("T2", bound=number_t)
 
@@ -60,16 +60,18 @@ __all__ = ("PHI",
            "optimize_parameter",
            "ispowerof2",
            "nextpowerof2",
-           "Histogram"
            )
 
 
+class NotFoundError(ValueError): pass
+
+
 # phi, in float (double) form and as Rational number with a precission of 2000
 # iterations in the fibonacci row (fib[2000] / fib[2001])
 PHI = 0.6180339887498949
 
 
-def intersection(u1:T, u2:T, v1:T, v2:T) -> tuple[T, T] | None:
+def intersection(u1: T, u2: T, v1: T, v2: T) -> tuple[T | None, T]:
     """
     return the intersection of (u1, u2) and (v1, v2) or None if no intersection
 
@@ -80,21 +82,29 @@ def intersection(u1:T, u2:T, v1:T, v2:T) -> tuple[T, T] | None:
         v2: higher bound of range V
 
     Returns:
-        the intersection between range U and range V, or None if
-        there is no intersection
+        the intersection between range U and range V as a tuple (start, end). If no intersection is found,
+        start will be None.
 
     Example::
 
-        >>> if intersec := intersection(0, 3, 2, 5):
-        ...     x0, x1 = intersec
+        >>> x0, x1 = intersection(0, 3, 2, 5)
+        >>> if x0 is not None:
+        ...     # there is an intersection
 
     """
     x0 = u1 if u1 > v1 else v1
     x1 = u2 if u2 < v2 else v2
-    return (x0, x1) if x0 < x1 else None
+    return (x0, x1) if x0 < x1 else (None, 0)
 
 
-def frange(start: float, stop: float=None, step: float=None) -> Iterator[float]:
+def hasintersect(u1: T, u2: T, v1: T, v2: T) -> bool:
+    """
+    True if the intervals (u1, u2) and (v1, v2) intersect
+    """
+    return u2 > v1 if u1 < v1 else v2 > u1
+
+
+def frange(start: float, stop: float = None, step: float = 0.) -> Iterator[float]:
     """
     Like xrange(), but returns list of floats instead
 
@@ -118,7 +128,7 @@ def frange(start: float, stop: float=None, step: float=None) -> Iterator[float]:
     if stop is None:
         stop = float(start)
         start = 0.0
-    if step is None:
+    if not step:
         step = 1.0
     numiter = int((stop - start) / step)
     for i in range(numiter):
@@ -153,7 +163,7 @@ def fraction_range(start: number_t, stop: number_t = None, step: number_t = None
         startF += step
 
 
-def linspace(start: float, stop: float, numitems: int) -> List[float]:
+def linspace(start: float, stop: float, numitems: int) -> list[float]:
     """ Similar to numpy.linspace, returns a python list """
     dx = (stop - start) / (numitems - 1)
     return [start + dx*i for i in range(numitems)]
@@ -286,7 +296,7 @@ def euclidian_distance(values: Sequence[float], weights: Sequence[float]=None) -
     return sqrt(sum(value**2 for value in values))
 
 
-def weighted_euclidian_distance(pairs: List[tuple[float, float]]) -> float:
+def weighted_euclidian_distance(pairs: list[tuple[float, float]]) -> float:
     """
     Reduces distances in multiple dimensions to 1 dimension.
 
@@ -331,7 +341,7 @@ def harmonic_mean(numbers: Sequence[T]) -> T:
 
 
 def split_interval_at_values(start: T, end: T, offsets: Sequence[T]
-                             ) -> List[tuple[T, T]]:
+                             ) -> list[tuple[T, T]]:
     """
     Split interval (start, end) at the given offsets
 
@@ -344,9 +354,12 @@ def split_interval_at_values(start: T, end: T, offsets: Sequence[T]
         a list of (start, end) segments where no segment extends over any
         of the given offsets
 
+
     Example
     ~~~~~~~
 
+    .. code::
+
         >>> split_interval_at_values(1, 3, [1.5, 2])
         [(1, 1.5), (1.5,  2), (2, 3)]
 
@@ -402,7 +415,7 @@ def logrange(start: float, stop: float, num=50, base=10) -> np.ndarray:
 
 
 def randspace(begin: float, end: float, numsteps: int, include_end=True
-              ) -> List[float]:
+              ) -> list[float]:
     """
     go from begin to end in numsteps at randomly spaced steps
 
@@ -490,12 +503,14 @@ def roundres(x, resolution=1.0):
     Round x with given resolution
 
     Example
-    ~~~~~~~
+    -------
 
-    >>> roundres(0.4, 0.25)
-    0.5
-    >>> roundres(1.3, 0.25)
-    1.25
+    .. code:: python
+
+        >>> roundres(0.4, 0.25)
+        0.5
+        >>> roundres(1.3, 0.25)
+        1.25
     """
     return round(x / resolution) * resolution
 
@@ -534,9 +549,10 @@ def modulo_shortest_distance(x: number_t, origin: number_t, mod: int):
     Example
     -------
 
-    Calculate the interval between two pitches, independently of octaves
+    Calculate the interval between note D5 and B3, independently of octaves
 
-    >>> interval = modulo_shortest_distance(n2m("D5"), n2m("B4"), 12)
+
+    >>> interval = modulo_shortest_distance(74, 59, 12)
     3
     """
     xclock = (x - origin) % mod
@@ -548,14 +564,15 @@ def modulo_shortest_distance(x: number_t, origin: number_t, mod: int):
 
 def rotate2d(point: tuple[float, float],
              degrees: float,
-             origin=(0, 0)) -> tuple[float, float]:
+             origin=(0, 0)
+             ) -> tuple[float, float]:
     """
     A rotation function that rotates a point around an origin
 
     Args:
-        point:   the point to rotate as a tuple (x, y)
+        point: the point to rotate as a tuple (x, y)
         degrees: the angle to rotate (counterclockwise)
-        origin:  the point acting as pivot to the rotation
+        origin: the point acting as pivot to the rotation
 
     Returns:
         the rotated point, as a tuple (x, y)
@@ -647,6 +664,9 @@ def optimize_parameter(func,
     """
     Optimize one parameter to arrive to a desired value.
 
+    Will raise NotFoundError if the maximum iterations are reached without
+    converging
+
     Args:
         func: a function returning a value which will be compared to `val`
         val: the desired value to arrive to
@@ -671,24 +691,33 @@ def optimize_parameter(func,
         1.25
 
     """
+    if val == 0:
+        raise ValueError("Cannot optimize for 0")
+
     param = paraminit
     for i in range(maxiterations):
         valnow = func(param)
-        relerror = abs(valnow - val) / valnow
+        relerror = abs(valnow - val) / val
         if relerror < maxerror:
             break
         if valnow > val:
             param = param * (1+relerror)
         else:
             param = param * (1-relerror)
-    return valnow, i
+    else:
+        valnow = func(param)
+        relerror = abs(valnow - val) / valnow
+        raise NotFoundError(f"The maximum iterations were reached without converging."
+                            f" Last param: {param}, value: {valnow}, relerror: {relerror}")
+
+    return param, i
 
 
 def intersection_area_between_circles(x1: float, y1: float, r1: float,
                                       x2: float, y2: float, r2: float
                                       ) -> float:
     """
-    Calculate the are of the intersection between two circles
+    Calculate the area of the intersection between two circles
 
     Args:
         x1: x coord of the center of the 1st circle
@@ -804,19 +833,3 @@ def nextpowerof2(x) -> int:
     return 1 if x == 0 else 2 ** (x - 1).bit_length()
 
 
-class Histogram:
-    def __init__(self, values: list[Number] | np.ndarray, numbins: int = 20):
-        _, edges = np.histogram(values, bins=numbins)
-        self.edges = edges
-        self.numbins = numbins
-        self.values = values
-        self._percentiles = np.linspace(0, 1, len(edges))
-
-    def valueToPercentile(self, value: float) -> float:
-        return np.interp(value, self.edges, self._percentiles)
-
-    def percentileToValue(self, percentile: float) -> float:
-        return np.interp(percentile, self._percentiles, self.edges)
-
-    def __repr__(self):
-        return f"Histogram(numbins={self.numbins}, edges={self.edges})"
